@@ -15,7 +15,7 @@ const bench = common.createBenchmark(main, {
     '/foo/bar/baz/asdf/quux.html',
     ['/foo/bar/baz/asdf/quux.html', '.html'].join('|')
   ],
-  n: [1e6]
+  n: [1e5]
 });
 
 function main({ n, pathext }) {
@@ -28,7 +28,7 @@ function main({ n, pathext }) {
 
   bench.start();
   for (var i = 0; i < n; i++) {
-    posix.basename(pathext, ext);
+    posix.basename(i % 3 === 0 ? `${pathext}${i}` : pathext, ext);
   }
   bench.end(n);
 }
