@@ -34,6 +34,7 @@ const {
   chunkExpression,
   httpSocketSetup,
   kIncomingMessage,
+  isLenient,
   _checkInvalidHeaderChar: checkInvalidHeaderChar
 } = require('_http_common');
 const { OutgoingMessage } = require('_http_outgoing');
@@ -47,6 +48,7 @@ const { IncomingMessage } = require('_http_incoming');
 const {
   ERR_HTTP_HEADERS_SENT,
   ERR_HTTP_INVALID_STATUS_CODE,
+  ERR_INVALID_ARG_TYPE,
   ERR_INVALID_CHAR
 } = require('internal/errors').codes;
 const Buffer = require('buffer').Buffer;
@@ -269,6 +271,8 @@ function writeHead(statusCode, reason, obj) {
   }
 
   this._storeHeader(statusLine, headers);
+
+  return this;
 }
 
 // Docs-only deprecated: DEP0063
@@ -287,6 +291,14 @@ function Server(options, requestListener) {
   this[kIncomingMessage] = options.IncomingMessage || IncomingMessage;
   this[kServerResponse] = options.ServerResponse || ServerResponse;
 
+  const insecureHTTPParser = options.insecureHTTPParser;
+  if (insecureHTTPParser !== undefined &&
+      typeof insecureHTTPParser !== 'boolean') {
+    throw new ERR_INVALID_ARG_TYPE(
+      'insecureHTTPParser', 'boolean', insecureHTTPParser);
+  }
+  this.insecureHTTPParser = insecureHTTPParser;
+
   net.Server.call(this, { allowHalfOpen: true });
 
   if (requestListener) {
@@ -302,7 +314,6 @@ function Server(options, requestListener) {
 
   this.timeout = 2 * 60 * 1000;
   this.keepAliveTimeout = 5000;
-  this._pendingResponseData = 0;
   this.maxHeadersCount = null;
   this.headersTimeout = 40 * 1000; // 40 seconds
 }
@@ -341,7 +352,9 @@ function connectionListenerInternal(server, socket) {
   socket.on('timeout', socketOnTimeout);
 
   var parser = parsers.alloc();
-  parser.reinitialize(HTTPParser.REQUEST, parser[is_reused_symbol]);
+  parser.reinitialize(HTTPParser.REQUEST, parser[is_reused_symbol],
+                      server.insecureHTTPParser === undefined ?
+                        isLenient() : server.insecureHTTPParser);
   parser.socket = socket;
 
   // We are starting to wait for our headers.
@@ -500,13 +513,15 @@ function onParserExecute(server, socket, parser, state, ret) {
   onParserExecuteCommon(server, socket, parser, state, ret, undefined);
 }
 
+const noop = () => {};
 const badRequestResponse = Buffer.from(
-  `HTTP/1.1 400 ${STATUS_CODES[400]}${CRLF}${CRLF}`, 'ascii'
+  `HTTP/1.1 400 ${STATUS_CODES[400]}${CRLF}` +
+  `Connection: close${CRLF}${CRLF}`, 'ascii'
 );
 function socketOnError(e) {
   // Ignore further errors
   this.removeListener('error', socketOnError);
-  this.on('error', () => {});
+  this.on('error', noop);
 
   if (!this.server.emit('clientError', e, this)) {
     if (this.writable) {
@@ -740,7 +755,7 @@ function resetHeadersTimeoutOnReqEnd() {
   const parser = this.socket.parser;
   // Parser can be null if the socket was destroyed
   // in that case, there is nothing to do.
-  if (parser !== null) {
+  if (parser) {
     parser.parsingHeadersStart = nowDate();
   }
 }
