@@ -47,6 +47,7 @@ const {
 } = constants;
 
 let statusMessageWarned = false;
+let statusConnectionHeaderWarned = false;
 
 // Defines and implements an API compatibility layer on top of the core
 // HTTP/2 implementation, intended to provide an interface that is as
@@ -60,6 +61,8 @@ function assertValidHeader(name, value) {
     err = new ERR_HTTP2_PSEUDOHEADER_NOT_ALLOWED();
   } else if (value === undefined || value === null) {
     err = new ERR_HTTP2_INVALID_HEADER_VALUE(value, name);
+  } else if (!isConnectionHeaderAllowed(name, value)) {
+    connectionHeaderMessageWarn();
   }
   if (err !== undefined) {
     Error.captureStackTrace(err, assertValidHeader);
@@ -90,6 +93,23 @@ function statusMessageWarn() {
   }
 }
 
+function isConnectionHeaderAllowed(name, value) {
+  return name !== constants.HTTP2_HEADER_CONNECTION ||
+         value === 'trailers';
+}
+
+function connectionHeaderMessageWarn() {
+  if (statusConnectionHeaderWarned === false) {
+    process.emitWarning(
+      'The provided connection header is not valid, ' +
+      'the value will be dropped from the header and ' +
+      'will never be in use.',
+      'UnsupportedWarning'
+    );
+    statusConnectionHeaderWarned = true;
+  }
+}
+
 function onStreamData(chunk) {
   const request = this[kRequest];
   if (request !== undefined && !request.push(chunk))
@@ -541,6 +561,11 @@ class Http2ServerResponse extends Stream {
   [kSetHeader](name, value) {
     name = name.trim().toLowerCase();
     assertValidHeader(name, value);
+
+    if (!isConnectionHeaderAllowed(name, value)) {
+      return;
+    }
+
     this[kHeaders][name] = value;
   }
 
@@ -568,16 +593,25 @@ class Http2ServerResponse extends Stream {
     if (this[kStream].headersSent)
       throw new ERR_HTTP2_HEADERS_SENT();
 
+    if (this.stream.destroyed)
+      return this;
+
     if (typeof statusMessage === 'string')
       statusMessageWarn();
 
     if (headers === undefined && typeof statusMessage === 'object')
       headers = statusMessage;
 
-    if (typeof headers === 'object') {
+    var i;
+    if (Array.isArray(headers)) {
+      for (i = 0; i < headers.length; i++) {
+        const header = headers[i];
+        this[kSetHeader](header[0], header[1]);
+      }
+    } else if (typeof headers === 'object') {
       const keys = Object.keys(headers);
       let key = '';
-      for (var i = 0; i < keys.length; i++) {
+      for (i = 0; i < keys.length; i++) {
         key = keys[i];
         this[kSetHeader](key, headers[key]);
       }
@@ -585,6 +619,8 @@ class Http2ServerResponse extends Stream {
 
     state.statusCode = statusCode;
     this[kBeginSend]();
+
+    return this;
   }
 
   write(chunk, encoding, cb) {
@@ -614,7 +650,7 @@ class Http2ServerResponse extends Stream {
 
     if ((state.closed || state.ending) &&
         state.headRequest === stream.headRequest) {
-      return false;
+      return this;
     }
 
     if (typeof chunk === 'function') {
