@@ -281,7 +281,7 @@ function inherits(ctor, superCtor) {
 
   if (superCtor.prototype === undefined) {
     throw new ERR_INVALID_ARG_TYPE('superCtor.prototype',
-                                   'Function', superCtor.prototype);
+                                   'Object', superCtor.prototype);
   }
   ctor.super_ = superCtor;
   Object.setPrototypeOf(ctor.prototype, superCtor.prototype);
